package com.example.module;

import java.io.UnsupportedEncodingException;
import java.nio.channels.SelectableChannel;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.rscja.deviceapi.Barcode1D;
import com.rscja.deviceapi.Infrared;
import com.rscja.deviceapi.Module;
import com.rscja.deviceapi.exception.ConfigurationException;
import com.rscja.utility.StringUtility;

import android.support.v4.app.Fragment;
import android.text.TextUtils;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;
import android.os.Build;

/**
 * 模块使用demo
 * 
 * 1、使用前请确认您的机器已安装此模块。
 * 2、要正常使用模块需要在\libs\armeabi\目录放置libDeviceAPI.so文件，同时在\libs\目录下放置DeviceAPI
 * .jar文件。 3、在操作设备前需要调用 init() 打开设备，使用完后调用 free() 关闭设备
 * 
 * 更多函数的使用方法请查看API说明文档
 * 
 * @author liuruifeng 2014年8月6日
 */
public class MainActivity extends Activity {

	private final static String TAG = "MainActivity";

	private Button btn_Start;
	private Button btn_Clear;
	private EditText et_fasong;
	private TextView tv_jieshou;
	private CheckBox cbHex;

	private Handler handler;
	private Thread thread;
	private ScrollView svResult;

	private boolean threadStop = true;

	private boolean isOpened = false;

	private Module mInstance;

	private boolean isHex = false;

	static final String s013 = "$013=";
	static final String s300 = "$300=";
	static final String obd_tt = "$OBD-TT";
	static final String est527 = "$EST527";
	static final String obd_start = "System running";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		init();

	}

	private void init() {
		btn_Start = (Button) findViewById(R.id.btn_Start);
		btn_Clear = (Button) findViewById(R.id.btn_Clear);
		et_fasong = (EditText) findViewById(R.id.et_fasong);
		svResult = (ScrollView) findViewById(R.id.scrollView1);
		cbHex = (CheckBox) findViewById(R.id.cbHex);
		tv_jieshou = (TextView) findViewById(R.id.tv_jieshou);

		try {
			mInstance = Module.getInstance();

		} catch (ConfigurationException e) {

			Toast.makeText(MainActivity.this, R.string.rfid_mgs_error_config,
					Toast.LENGTH_SHORT).show();

			return;
		}

		btn_Start.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {

				send();

			}
		});

		btn_Clear.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				clear();

			}
		});

		handler = new Handler() {

			@Override
			public void handleMessage(Message msg) {
				if (msg != null) {

					String strData = msg.obj.toString();

					tv_jieshou.append(strData);

					scrollToBottom(svResult, tv_jieshou);
				}
			}

		};

		cbHex.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView,
					boolean isChecked) {

				isHex = isChecked;

			}
		});

	}

	private void send() {

		String data = et_fasong.getText().toString();

		if (StringUtility.isEmpty(data)) {
			Toast.makeText(MainActivity.this, R.string.ping_msg_not_null,
					Toast.LENGTH_SHORT).show();
			return;
		}

		byte[] sendBytes = null;

		if (isHex) {
			sendBytes = StringUtility.hexString2Bytes(data);

		} else {
			byte[] bdata = data.getBytes();

			sendBytes = new byte[bdata.length + 2];
			for (int i = 0; i < bdata.length; i++) {
				sendBytes[i] = bdata[i];

			}
			sendBytes[sendBytes.length - 2] = '\r';
			sendBytes[sendBytes.length - 1] = '\n';

			Log.i("MainActivity", "sendBytes = " + arrayToString(sendBytes));

			// try {
			// sendBytes=data.getBytes("iso-8859-1");
			// } catch (UnsupportedEncodingException e) {
			// // TODO Auto-generated catch block
			// e.printStackTrace();
			// }

		}

		if (mInstance.send(sendBytes)) {
			Toast.makeText(MainActivity.this, "发送成功", Toast.LENGTH_SHORT)
					.show();

		} else {
			Toast.makeText(MainActivity.this, "发送失败", Toast.LENGTH_SHORT)
					.show();

		}

	}

	public final String arrayToString(byte[] bytes) {
		StringBuffer buff = new StringBuffer();
		for (int i = 0; i < bytes.length; i++) {
			buff.append(bytes[i] + " ");
		}
		return buff.toString();
	}

	private void clear() {
		tv_jieshou.setText("");

	}

	class ReceiveThread extends Thread {
		byte[] data;
		Message msg;

		@Override
		public void run() {

			do {

				data = mInstance.receive();

				msg = new Message();

				if (data == null || data.length == 0) {
					continue;

				} else {

					msg.arg1 = 1;

					if (isHex) {
						msg.obj = StringUtility.bytes2HexString(data,
								data.length);
					} else {
						msg.obj = proccData(new String(data));
					}
				}

				handler.sendMessage(msg);

			} while (!threadStop);
		}
	}

	private String proccData(String data) {
		if (TextUtils.isEmpty(data)) {
			return null;
		}

		String[] datas = data.split(",");

		if (datas == null || datas.length == 0) {
			return data;
		}

		if (datas[0].contains(s013)) {
			return "当前车速：" + datas[0].replace(s013, "") + " km/h\n";
		} else if (datas[0].contains(s300)) {
			if (datas.length > 2) {
				return "总里程：" + datas[2] + " km\n";
			} else {
				return data;
			}
		}else if (datas[0].contains(obd_tt)) {
			SharedPreferences sp=getPreferences(MODE_PRIVATE);
			
			Editor edt= sp.edit();
			edt.putLong("flameout", System.currentTimeMillis());
			edt.commit();
			
			
		}
		else if (datas[0].contains(est527)) {
			SharedPreferences sp=getPreferences(MODE_PRIVATE);
			long flameout=sp.getLong("flameout", 0);
			if(flameout!=0)
			{
				long time=System.currentTimeMillis()-flameout;
				return datas[0]+"\n"+"距离上次熄火：" + (time/1000) + " s\n";
				
			}
			

		}

		return data;
	}

	/**
	 * 设备上电异步类
	 * 
	 * @author liuruifeng
	 */
	public class InitTask extends AsyncTask<String, Integer, Boolean> {
		ProgressDialog mypDialog;

		@Override
		protected Boolean doInBackground(String... params) {
			// TODO Auto-generated method stub
			// return mInstance.SerailOpen(2,9600,0);
			return mInstance.init(5, 9600);
		}

		@Override
		protected void onPostExecute(Boolean result) {
			super.onPostExecute(result);
			isOpened = result;
			mypDialog.cancel();

			if (!result) {
				Toast.makeText(MainActivity.this, "init fail",
						Toast.LENGTH_SHORT).show();
			}
		}

		@Override
		protected void onPreExecute() {
			// TODO Auto-generated method stub
			super.onPreExecute();

			mypDialog = new ProgressDialog(MainActivity.this);
			mypDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
			mypDialog.setMessage("init...");
			mypDialog.setCanceledOnTouchOutside(false);
			mypDialog.show();
		}

	}

	@Override
	protected void onResume() {
		super.onResume();

		new InitTask().execute();
		threadStop = false;

		new ReceiveThread().start();

	}

	@Override
	protected void onPause() {
		super.onPause();

		threadStop = true;

		if (isOpened) {
			// mInstance.SerailClose();
			mInstance.free();
		}

	}

	/**
	 * 将ScrollView根据内部控件高度定位到底端
	 * 
	 * @param scroll
	 * @param inner
	 */
	public void scrollToBottom(final View scroll, final View inner) {

		Handler mHandler = new Handler();

		mHandler.post(new Runnable() {
			public void run() {
				if (scroll == null || inner == null) {
					return;
				}
				int offset = inner.getMeasuredHeight() - scroll.getHeight();
				if (offset < 0) {
					offset = 0;
				}

				scroll.scrollTo(0, offset);
			}
		});
	}

}
